#!/bin/sh
#********************************************************************************
# (c) Copyright [2013-2021] Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates 
# and licensors (Micro Focus) are as may be set forth in the express warranty 
# statements accompanying such products and services. Nothing herein should be 
# construed as constituting an additional warranty. Micro Focus shall not be liable 
# for technical or editorial errors or omissions contained herein. The information 
# contained herein is subject to change without notice.
#********************************************************************************

#Include library
. /opt/novell/iprintmobile/bin/lib.sh

#Return Code
RVAL=0

#Internal version
VERSION="1.1"

function Usage()
{
  echo "This tool allows configuration of printer driver to be used for document rendering."
  echo "You can configure a global default driver for all printers and also a printer specific driver."
  echo "Usage: $0 -c <CUPS Printer Name> -f <Driver file Path>  -p <Mobile Printer Name> | -d | -s |-r <CUPS Printer Name>"
  echo "Usage: $0 -U <Default Driver URI>"
  echo ""
  echo "-c   - name of the CUPS dummy printer to be created"
  echo "-f   - path to the driver file to be used"
  echo "-p   - specify printer name to which this driver should be assigned or use -d for default"
  echo "-d   - set this driver as the default or use -p to specify printer name"
  echo "-r   - name of the CUPS dummy printer to be removed"
  echo "-s   - Creates or removes a standalone printer "
  echo "-U   - To set the default driver from the appliance script. $DEFAULT_DRIVER will be used. The cups printer name will be $DEFAULT_PRINTER_NAME"
  echo "-h   - displays help"
  exit 1
}


echo
echo "---------------------------------------------------"
echo "$SERVICE_NAME Driver Configuration Tool $VERSION"
echo "---------------------------------------------------"
#echo "Start Time: "`date`
echo

CUPS_PRINTER_NAME=""
DRIVER_FILE=""
PRINTER_NAME=""
REMOVE_CUPS_PRINTER=""
STANDALONE_CUPS=""
DRIVERURI=""
while getopts c:p:f:r:U:hds o
do
        case "$o" in
                c)      CUPS_PRINTER_NAME="$OPTARG";;

		r)      CUPS_PRINTER_NAME="$OPTARG"
			DRIVER_FILE="-"
			PRINTER_NAME="-"
			REMOVE_CUPS_PRINTER="true";;
				
                p)      
                        if [ -z "$PRINTER_NAME" ]; then
                          PRINTER_NAME="$OPTARG"
                        else
                          echo "Invalid command, use either one of the -p and -d options."
                          Usage
                        fi
                        ;;

                f)      DRIVER_FILE="$OPTARG";;

                d)      
                        if [ -z "$PRINTER_NAME" ]; then
                          PRINTER_NAME="--default"
                        else
                          echo "Invalid command, use either one of the -p and -d options."
                          Usage
                        fi
                        ;;
		s)   	PRINTER_NAME="-"
			STANDALONE_CUPS="true";;
			# If the option U is present, it is assumed that the operation is setting default driver
		U) 	DRIVERURI="$OPTARG"
			CUPS_PRINTER_NAME="$DEFAULT_PRINTER_NAME"
			DRIVER_FILE="$DEFAULT_DRIVER"
			PRINTER_NAME="--default";;
                h)      Usage;;

                *)      Usage;;
        esac
done

if [ -z "$CUPS_PRINTER_NAME" -o -z "$DRIVER_FILE" -o -z "$PRINTER_NAME" ]; then
  echo "$ERROR Invalid command."
  Usage
fi

#STANDALONE_CUPS parameter is used to create a standalone cups printer by mobile server code. 
#Checking for mobile server status is not needed in case of creating standalone cups printer
if test -z "$STANDALONE_CUPS"
then
#Check iPrint Mobile Server
rcnovell-iprint-mobile status > /dev/null 2>&1
if [ $? -eq 0 ] ; then
  echo "$FATAL $SERVICE_NAME service is running. Exiting."
  exit $MOBILE_SERVER_RUNNING
fi
fi 

#In case of no arguments
#Default driver : default_driver.ppd
#Default printer name : DEFAULT

systemctl status $IPRINT_CUPS_SERVICE > /dev/null 2>&1
if [ $? -ne 0 ] ; then
  echo "$WARN CUPS service is not running. Starting..."
  systemctl start $IPRINT_CUPS_SERVICE
  systemctl status $IPRINT_CUPS_SERVICE
  if [ $? -ne 0 ] ; then
    echo "$FATAL Failed to start CUPS service."
    exit $CUPS_ERROR
  fi
fi

#Check whether script is called for removing CUPS printer
if test -n "$REMOVE_CUPS_PRINTER"; then
	
	OPTION="n"
	#for creating standalone cups , setting the option to y
	if test -n "$STANDALONE_CUPS" ;  then
		OPTION="y"
	else
		echo -n "This will delete the CUPS printer and its association with all the mobile printers. Do you want to continue? (y/n): "
		read OPTION
	fi
	if [ "$OPTION" == "y" -o "$OPTION" == "Y" ]; then
			
		#Removing Printer from CUPS
		RemoveCUPSPrinter "$CUPS_PRINTER_NAME"
		RVAL=$?

		if [ $RVAL -ne 0 ] ; then
		  echo "$FATAL Failed to remove CUPS printer (error: $RVAL)." >> "${ERR_LOG}"
		  exit $RVAL
		fi
		
		echo "$CUPS_PRINTER_NAME removed successfully." | tee -a "${ERR_LOG}"
	fi
	exit 0
fi

CreateCUPSPrinter "$CUPS_PRINTER_NAME" "$DRIVER_FILE"
RVAL=$?

if [ $RVAL -ne 0 ] ; then
  echo "$FATAL Failed to add the driver to CUPS. (Error - $RVAL)."
  exit $RVAL
fi

if test -n "$STANDALONE_CUPS" ; then
  exit $RVAL
fi


touch "$DRIVERS_FILE"
echo "id=$CUPS_PRINTER_NAME" >> "$DRIVERS_FILE"
echo "uri=$DRIVERURI" >> "$DRIVERS_FILE"
chmod 755 "$DRIVERS_FILE"
chown root:iprint "$DRIVERS_FILE"
RVAL=$?

#echo "End Time: "`date`

exit $RVAL
