#!/bin/sh
#********************************************************************************
# Copyright 2013-2023 Open Text.
#
# The only warranties for products and services of Open Text and its affiliates and licensors (“Open Text”)
# are as may be set forth in the express warranty statements accompanying such products and services.
# Nothing herein should be construed as constituting an additional warranty. Open Text shall not be liable
# for technical or editorial errors or omissions contained herein. The information contained herein is
# subject to change without notice.
#********************************************************************************
SERVICE_NAME="iPrint Mobile Server"
VERSION="2.0"

APACHE="/usr/sbin/rcapache2"
TOMCAT="/usr/sbin/rctomcat"
TOMCAT_BIN_NAME="rctomcat"
TOMCAT_SERVER_XML="/usr/share/tomcat/conf/server.xml"
APACHE_CONF_FILE="/etc/apache2/conf.d/iprint_mobile_apache.conf"
DEFAULT_AJP_PORT=8009
CUSTOM_AJP_PORT=18009
JAVA=""
DEBUG="DEBUG:"
INFO="INFO:"
WARN="WARNING:"
ERROR="ERROR:"
FATAL="FATAL:"
CONF_PATH="/etc/opt/novell/iprintmobile/conf/"
CONF_FILE="/etc/opt/novell/iprintmobile/conf/iprintmobile.conf"
XMLDB_FILE="/var/opt/novell/iprintmobile/iprintmobiledb.xml"
DRIVERS_FILE="/var/opt/novell/iprintmobile/driver.properties"
LIB_PATH="/opt/novell/iprintmobile/lib"
BIN_PATH="/opt/novell/iprintmobile/bin"
IPRINT_MOBILE_JAR="$LIB_PATH/iPrintMobile.jar"
DRIVERS_DIR="/var/opt/novell/iprintmobile/drivers"
#Since the default driver at /var/opt/novell/iprintmobile/drivers/default_driver.ppd is uploaded to the driver store, the name of the cups printer to
#be created is URL encoded
DEFAULT_PRINTER_NAME="Generic+PostScript+Printer+Foomatic%2FPostscript+%28recommended%29"
DEFAULT_DRIVER="$DRIVERS_DIR/default_driver.ppd"
CUPS_CONF_FILE="/etc/cups/cupsd.conf"
IPRINT_CUPS_PATH="/etc/opt/novell/iprintmobile/cups"
IPRINT_CUPS_CONF_FILE="/etc/opt/novell/iprintmobile/cups/cupsd-iprint.conf"
IPRINT_CUPSFILES_CONF_FILE="/etc/opt/novell/iprintmobile/cups/cups-files.conf"
IPRINT_CUPS_SERVICE="iprint-cups.service"
ERR_LOG="/var/opt/novell/log/iprintmobile/iprintmobile_config.log"
DEFAULT_CUPS_PORT=3017
#A Javascript file where end user related iPrint mobile configuration is published.
#for example, global email id, printer list
CONF_PUBLISH_FILE="iprintmobileconf.js"
CONF_PUBLISH_DIR="/srv/tomcat/webapps/iprint/app/ui"

KEYTOOL="/usr/lib64/jvm/jre-11-openjdk/bin/keytool"
OPENSSL="/usr/bin/openssl"

#Renderer configuration 
RENDERER_TIME_OUT_KEY="renderer_time_out"

#Arguments
HELP_OPTION_1="--help"
HELP_OPTION_2="-h"
NO_HEADER_OPTION="--noheader"

#Error Codes
JAVA_ERROR=3
CONFIG_ERROR=5
INSTALL_ERROR=6
CUPS_CONFIG_ERROR=7
CUPS_ERROR=8
CUPS_CREATE_ERROR=9
MOBILE_SERVER_RUNNING=10
WGET_ERROR=11
IPP_PARSE_ERROR1=12
IPP_PARSE_ERROR2=13
IPP_PARSE_ERROR3=14
JAVA_VERSION_ERROR=15
CONF_PUBLISH_ERROR=16
TOMCAT_ERROR=17
LIST_RENDERER_ERROR=19
REMOVE_RENDERER_ERROR=20;
CONFIG_EMAIL_ERROR=21;
CONFIG_PRINT_OPTION_ERROR=22;
REMOVE_PRINTER_ERROR=24;
CONFIG_DISABLE_EMAIL_ERROR=25;
CUPS_PRINTER_REMOVAL_ERROR=26

CheckTomcat()
{
  #Check tomcat path
  TOMCAT=`which $TOMCAT_BIN_NAME 2>&1`
  if [ $? -ne 0 ]; then
    return $TOMCAT_ERROR
  fi
  return 0
}

getValueFromCredStore()
{
	/usr/bin/base64 -di  /etc/opt/novell/iprintmobile/conf/credstore.conf | grep $1 | cut -f2 -d\> | cut -f1 -d\<
}

#List all available renderers
listAvailableRenderers()
{
	/bin/sh "$BIN_PATH/iprintmobiledb.sh" list_renderers  --noheader
	
	if [ $? -ne 0 ] ; then
       echo "$INFO No renderer has been registered." | tee -a "${ERR_LOG}"
       return $LIST_RENDERER_ERROR
     fi
}

#Unregister a renderer by RENDERER_ID
unregisterRendererByID()
{
	/bin/sh "$BIN_PATH/iprintmobiledb.sh" remove_renderer $1 --noheader
	
	if [ $? -ne 0 ] ; then
       echo "$FATAL Un-registering Renderer failed."  | tee -a "${ERR_LOG}"
       return $REMOVE_RENDERER_ERROR
	else
		echo "$INFO Renderer un-registered successfully." | tee -a "${ERR_LOG}"
		return 0
     fi
}

#Unregister a printer by PRINTER_ID
unregisterPrinterByID()
{
	/bin/sh "$BIN_PATH/iprintmobiledb.sh" delete_printer $1 --noheader
	
	if [ $? -ne 0 ] ; then
       echo "$FATAL Removing Printer failed."  | tee -a "${ERR_LOG}"
       return $REMOVE_PRINTER_ERROR
	else
		echo "$INFO Printer deleted successfully." | tee -a "${ERR_LOG}"
		return 0
     fi
}

#Saves the given key value pair in to the configuration file
#$1 - Key
#$2 - Value
SaveConfig()
{
  #Check if the config file exists
  if [ ! -f $CONF_FILE ] ; then
    #Copy the template as configuration file
    if [ ! -f $CONF_FILE.template ] ; then
      echo "$FATAL Failed to find the template configuration file. Corrupt installation, re-install again." | tee -a "${ERR_LOG}"
      return $INSTALL_ERROR
    else
      cp $CONF_FILE.template $CONF_FILE
    fi
  fi

  if [ -f $CONF_FILE ] ; then
    `sed -i "s/$1=.*/$1=$2/" $CONF_FILE`
     if [ $? -ne 0 ] ; then
       echo "$FATAL Failed to update the configuration file: $CONF_FILE"  | tee -a "${ERR_LOG}"
       return $CONFIG_ERROR
     fi
  else
    # This should not happen.
    echo "$FATAL Failed to find the $CONF_FILE." | tee -a "${ERR_LOG}"
    return $CONFIG_ERROR
  fi
}

#Check if cups running
#returns 0 if its running.
CupsCheck()
{
  systemctl status $IPRINT_CUPS_SERVICE >> "${ERR_LOG}" 2>&1
  return $?
}

#Check if its an OES15 server
#returns 0 if its an OES Server
IsOESServer()
{
  #cat /etc/novell-release | grep "Open Enterprise Server 11"  >> "${ERR_LOG}" 2>&1
  # changing oes release to 2015
  cat /etc/novell-release | grep "Open Enterprise Server 2015"  >> "${ERR_LOG}" 2>&1
  return $?
}

#Check if its an iPrint Server
#returns 0 if its an iPrint Server
IsiPrintServer()
{
  if [ -f /etc/opt/novell/iprint/conf/ipsmd-template.conf ]; then
    return 0
  else
    return 1
  fi
}

#Check if there is a port conflict with Apache 631 for iPrint Server
#returns 0 if there is no conflict
CheckCupsConflict()
{
  IsiPrintServer

  if [ $? -eq 0 ] ; then
    #Check if CUPS is also configured on the same port.
    grep "^Listen.*:631" $IPRINT_CUPS_CONF_FILE > /dev/null 2>&1
    if [ $? -eq 0 ] ; then
      return 1
    else
      return 0
    fi
  fi

  return 0;
}

#Saves the given port number to cups config file
#$1 - Port number
SaveCUPSPort()
{
  if [ -f $IPRINT_CUPS_CONF_FILE ] ; then
     sed -i "/^Listen/s/:631 *$/:$1/" $IPRINT_CUPS_CONF_FILE
     if [ $? -ne 0 ] ; then
       echo "$FATAL Failed to update the configuration file '$IPRINT_CUPS_CONF_FILE'."  | tee -a "${ERR_LOG}"
       return $CUPS_CONFIG_ERROR
     fi
  else
    # This should not happen.
    echo "$FATAL Failed to find the $CUPS_CONF_FILE." | tee -a "${ERR_LOG}"
    return $CUPS_CONFIG_ERROR
  fi

  return 0;
}

#This method creates two printers one for color and other for grey scale
#$1 - Printer Name
#$2 - Driver Name
#returns 0 on success
CreateCUPSPrinter()
{
  #Check Driver
  if [ -f "$2" ] ; then
    cp "$2" /tmp/driver.ppd >> "${ERR_LOG}" 2>&1
    cp "$2" /tmp/driver_color.ppd >> "${ERR_LOG}" 2>&1
    grep -i "ColorDevice:" /tmp/driver.ppd >/dev/null 2>&1
    if [ $? -ne 0 ] ; then
      #This is strange but lets add it to both
      echo "*ColorDevice:   False" >> /tmp/driver.ppd
      echo "*ColorDevice:   True" >> /tmp/driver_color.ppd  
    else
      sed -i "s/^*ColorDevice:.*/*ColorDevice:   False/" /tmp/driver.ppd
      sed -i "s/^*ColorDevice:.*/*ColorDevice:   True/" /tmp/driver_color.ppd
    fi
	#Setting Language level to 1 [GrayScale]
	#This setting is same for both PS and PCL drivers
	grep -i "LanguageLevel:" /tmp/driver.ppd >/dev/null 2>&1
    if [ $? -ne 0 ] ; then
      #This is strange but lets add it to both
      echo "*LanguageLevel:   \"1\"" >> /tmp/driver.ppd
    else
      sed -i "s/^*LanguageLevel:.*/*LanguageLevel:   \"1\"/" /tmp/driver.ppd
    fi
  else
    echo "$FATAL Could not find the driver '$2'." | tee -a "${ERR_LOG}"
    return $CUPS_CREATE_ERROR
  fi
  #printfile:/drivers/spool is a dummy backend name for specifying the deviceURI . 
  # The deviceURL name should be same as the file name . for eg: printfile is the name of the backend file added in /usr/lib64/cups/backend
  /usr/sbin/lpadmin -p "$1" -E -v printfile:/drivers/spool/  -P /tmp/driver.ppd >> "${ERR_LOG}" 2>&1
  LIB_RVAL=$?
  /usr/sbin/lpadmin -p "$1_COLOR" -E -v printfile:/drivers/spool/  -P /tmp/driver_color.ppd  >> "${ERR_LOG}" 2>&1
  LIB_RVAL=$(($LIB_RVAL|$?))

  #Remove temporary drivers
  rm /tmp/driver.ppd >> "${ERR_LOG}" 2>&1
  rm /tmp/driver_color.ppd >> "${ERR_LOG}" 2>&1

  if [ $LIB_RVAL -ne 0 ] ; then
    echo "$FATAL Failed to add printer driver (error: $LIB_RVAL)." | tee -a "${ERR_LOG}"
    return $CUPS_CREATE_ERROR
  fi
  
  return 0
}

#This method is used to remove the CUPS printer created for Mobile Printing
#$1 - Driver Name
#returns 0 on success
RemoveCUPSPrinter()
{
  LIB_RVAL=0
  # Using lpadmin tool to remove the CUPS printer
  #Checking for the existance of the CUPS printer.
  CheckCupsPrinter "$1_COLOR"
  if [ $? -eq 0 ] ; then
    # Syntax: lpadmin -x <CUPS printer name>
    /usr/sbin/lpadmin -x "$1_COLOR" >> "${ERR_LOG}" 2>&1
    LIB_RVAL=$?
  fi
  CheckCupsPrinter "$1"
  if [ $? -eq 0 ] ; then
    # Syntax: lpadmin -x <CUPS printer name>
    /usr/sbin/lpadmin -x "$1" >> "${ERR_LOG}" 2>&1
    LIB_RVAL=$(($LIB_RVAL|$?))
  fi
  if [ $LIB_RVAL -ne 0 ] ; then
    echo "$FATAL Failed to remove CUPS printer $1 | $1_COLOR (error: $LIB_RVAL)." | tee -a "${ERR_LOG}"
    return $CUPS_PRINTER_REMOVAL_ERROR
  fi
  
  return 0
}

#Check if cups printer is installed already
#returns 0 if its installed.
CheckCupsPrinter()
{
  /usr/bin/lpstat -p | grep "$1" >> "${ERR_LOG}" 2>&1
  return $?
}

CreateXMLDB()
{
  if [ ! -f $XMLDB_FILE ] ; then
    #Copy the template as database file
    if [ ! -f $XMLDB_FILE.template ] ; then
      echo "$FATAL Failed to find the template database file. Corrupt installation, re-install again." | tee -a "${ERR_LOG}"
      return $INSTALL_ERROR
    else
      cp $XMLDB_FILE.template $XMLDB_FILE
    fi
  else
    echo "$DEBUG Database already exists." >> "${ERR_LOG}"
  fi

  return 0
}

#Make given CUPS Printer as default for iPrint Mobile
#$1 - Default CUPS Printer name
#$2 - Mobile Printer
ConfigPrinterDriver()
{

#Checking if the driver URI is empty to see if this has been called from appliance initial script to set the default driver.
#$3 will be driver URI which is being called from config_driver.sh
  if [ -z "$3" ]; then
	/bin/sh "$BIN_PATH/iprintmobiledb.sh" add_driver "$1" "$2" "$NO_HEADER_OPTION"
  else
	/bin/sh "$BIN_PATH/iprintmobiledb.sh" add_driver "$1" "$2" "$3" "$NO_HEADER_OPTION"
  fi
  return $?
}

#Checking java
CheckJava()
{
  JAVA=`which java 2>&1`
  if [ $? -ne 0 ]; then
    echo "$FATAL Could not find Java in PATH."  | tee -a "${ERR_LOG}"
    return $JAVA_ERROR
  fi

  #Lets make sure that Java Vesion is >= 1.8
  #Get the current version
  CURRENT_VERSION=`$JAVA -version 2>&1 | grep "version" | awk -F"\"" '{print $2}' | awk -F"." '{print $1"."$2}'`
  RESULT=`echo "$CURRENT_VERSION >= 1.8" | bc -l`
  if [ $RESULT -eq 1 ]; then
    return 0
  else
    echo "$FATAL The detected Java version is less than 1.8. Ensure that the PATH variable points to Java having minimum version as 1.8." | tee -a "${ERR_LOG}"
    return $JAVA_VERSION_ERROR
  fi
}

#Remove given CUPS Printer from iPrint Mobile
#$1 - Default CUPS Printer name
RemovePrinterDriver()
{ 
  /bin/sh "$BIN_PATH/iprintmobiledb.sh" remove_driver "$1" "$NO_HEADER_OPTION"
  return $?
}

#Gets all the printers from given iPrint server and adds them
#to iPrint Mobile database
#$1 - Server address
#returns 0 on success
ConfigurePrinters()
{

  wget --timeout 30 $1/ipp/pdisplay.htm -O /tmp/iprint_printers >> "${ERR_LOG}" 2>&1
  LIB_RVAL=$?
  if [ $LIB_RVAL -ne 0 ] ; then
    echo "$ERROR Failed to get printer details from $1 (Error - $LIB_RVAL)." | tee -a "${ERR_LOG}"
    return $LIB_RVAL
  fi

#Searching for the Non-Secure printer by patterns and fetching it
  if [ -f "/tmp/iprint_printers" -a -s "/tmp/iprint_printers" ]; then
    grep "ipp://" /tmp/iprint_printers > /tmp/iprint_ippuri 2>&1
    LIB_RVAL=$?
    if [ $LIB_RVAL -ne 0 ] ; then
      echo "$ERROR Failed to process server response (Error - $LIB_RVAL)." | tee -a "${ERR_LOG}"
      return $IPP_PARSE_ERROR1
    fi  

    sed -i "s/.*ippPrinterURL=//" /tmp/iprint_ippuri >> "${ERR_LOG}" 2>&1
    LIB_RVAL=$?
    if [ $LIB_RVAL -ne 0 ] ; then
      echo "$ERROR Failed to process server response (Error - $LIB_RVAL)." | tee -a "${ERR_LOG}"
      return $IPP_PARSE_ERROR2
    fi  

    cat /tmp/iprint_ippuri | grep "iprint.gif\|direct.gif" | awk -F"&" '{print $1}' > /tmp/iprint_nonsecure_list
    LIB_RVAL=$?
    if [ $LIB_RVAL -ne 0 ] ; then
      echo "$ERROR Failed to process server response (Error - $LIB_RVAL)." | tee -a "${ERR_LOG}"
      return $IPP_PARSE_ERROR3
    else
      #Invoke config_printer.sh for bulk population
      if [ -s /tmp/iprint_nonsecure_list ] ; then
         /bin/sh "$BIN_PATH/config_printer.sh" -f /tmp/iprint_nonsecure_list >> "${ERR_LOG}" 2>&1
         LIB_RVAL=$?
         if [ $LIB_RVAL -ne 0 ] ; then
           return $LIB_RVAL
         fi
      fi
    fi

    cat /tmp/iprint_ippuri | grep "iprints.gif\|directs.gif" | awk -F"&" '{print $1}' > /tmp/iprint_secure_list
    LIB_RVAL=$?
    if [ $LIB_RVAL -ne 0 ] ; then
      echo "$ERROR Failed to process server response (Error - $LIB_RVAL)." | tee -a "${ERR_LOG}"
      return $IPP_PARSE_ERROR3
    else
      #Invoke config_printer.sh for bulk population
      if [ -s /tmp/iprint_secure_list ] ; then
         /bin/sh "$BIN_PATH/config_printer.sh" -f /tmp/iprint_secure_list >> "${ERR_LOG}" 2>&1
         LIB_RVAL=$?
         if [ $LIB_RVAL -ne 0 ] ; then
            return $LIB_RVAL
         fi
      fi
    fi 
 
    rm /tmp/iprint_printers /tmp/iprint_ippuri /tmp/iprint_secure_list /tmp/iprint_nonsecure_list >> "${ERR_LOG}" 2>&1
  else
    echo "$ERROR Failed to get printer details from $1 (Error - $LIB_RVAL)." | tee -a "${ERR_LOG}"
    return $WGET_ERROR
  fi
  return 0
}


isMobilityConfigured()
{
    # In case of cluster, we are checking whether pool is running on this node or not.
    # Assumption: Caller takes care of this scenario, Hence the following case might not occur.
    # Side effect: If caller doesn't check for pool before calling this function, caller can
    # start "Full configuration" of Mobile Server(MS) and  further scripts "MIGHT" fail because they will not
    # be able to find the configuration file which are on cluster but are not accessible from this node.  
    if [ ! -f $IPRINT_CONF ]; then
        echo $IPRINT_CONF "is not accessible. Pool might not be running on this node " >> ${ERR_LOG}
        return 0
    fi
    STATUS=$(cat /etc/opt/novell/iprint/conf/iprintconf.properties  | grep mobility_addon_configured | cut -d= -f2)
    if [ $STATUS == "true" ]; then
        echo "Mobility configured" >> "${ERR_LOG}"
        return 1
    else
        echo "Mobility is not configured" >> ${ERR_LOG}
        return 0
    fi
}

isCluster() 
{
    IPRINT_NSS=$(grep Required-Start /opt/novell/iprint/init.d/novell-ipsmd | grep nss)
    if [ ! -z "${IPRINT_NSS}" ]; then
        echo "Cluster setup" >> "${ERR_LOG}"
        return 1
    else
        echo "Not a cluster setup" >> "${ERR_LOG}"
        return 0
    fi
}

isPoolRunningOnThisNode()
{
	MEDIA_DIR=$(readlink /etc/opt/novell/iprint/conf)
    TEST_FILE="$MEDIA_DIR/iprintconf.properties"
    if [ -f $TEST_FILE ]; then
        echo "Pool is running on this node" >> ${ERR_LOG}
        return 1
    else
        echo "Pool is not running on this node" >> ${ERR_LOG}
        return 0
    fi
}