#!/bin/sh
#********************************************************************************
# (c) Copyright [2017-2021] Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates 
# and licensors (“Micro Focus”) are as may be set forth in the express warranty 
# statements accompanying such products and services. Nothing herein should be 
# construed as constituting an additional warranty. Micro Focus shall not be liable 
# for technical or editorial errors or omissions contained herein. The information 
# contained herein is subject to change without notice.
#********************************************************************************

ME=$(basename "$0")
IPRINT_CUPS=/etc/opt/novell/iprintmobile/cups
ERR_LOG="/var/opt/novell/log/iprintmobile/main_config_oes.log"
LOCAL_ERR_LOG="/var/opt/novell/log/iprintmobile_local/main_config_oes.log"

initLogs()
{
    if [ ! -d /var/opt/novell/log/iprintmobile/ ]; then
        ERR_LOG=$LOCAL_ERR_LOG
    fi
    echo "Log file: ${ERR_LOG}"  | tee -a  "${ERR_LOG}"
}

isCluster() 
{
    IPRINT_NSS=$(grep Required-Start /opt/novell/iprint/init.d/novell-ipsmd | grep nss)
    if [ ! -z "${IPRINT_NSS}" ]; then
        echo "Cluster setup"  >>  "${ERR_LOG}"
        return 1
    else
        echo "Not a cluster setup"  | tee -a  "${ERR_LOG}"
        return 0
    fi
}

isPoolRunningOnThisNode()
{
	MEDIA_DIR=$(readlink /etc/opt/novell/iprint/conf)
	TEST_FILE="$MEDIA_DIR/iprintconf.properties"
    if [ -f $TEST_FILE ]; then
        echo "Pool is running on this node"  >> tee -a  ${ERR_LOG}
        return 1
    else
        echo "Pool is not running on this node"  | tee -a  ${ERR_LOG}
        return 0
    fi
}

getVolumePath()
{
    VOL=$(readlink /etc/opt/novell/iprint/conf | awk -F/ '{print $(NF-5)}')
    echo "/media/nss/$VOL"
}

createCupsSymLink() 
{
    CUPS_PATH=$(getVolumePath)
    echo "Detected NSS volume path $CUPS_PATH" | tee -a ${ERR_LOG}
	
    mv $IPRINT_CUPS ${IPRINT_CUPS}_local
    ln -s $CUPS_PATH$IPRINT_CUPS $IPRINT_CUPS
}

fixCupsSymlink() 
{
    isCluster
    if [ $? -eq 1 ]; then
        echo "Checking cups symlink " >> $ERR_LOG
        if [ -L $IPRINT_CUPS ]; then
	        echo "$IPRINT_CUPS is a symlink "  >> $ERR_LOG
            isPoolRunningOnThisNode
            if [ $? -eq 1 ]; then
                if [ -e $IPRINT_CUPS ]; then
	            	echo "$IPRINT_CUPS is a valid symlink. Nothing to fix." | tee -a ${ERR_LOG}
        	    else
        		    echo "Pool is running but $IPRINT_CUPS directory is not accessible. This needs a fix." | tee -a ${ERR_LOG}
                    createCupsSymLink
            	fi
            fi
         else
            echo "$IPRINT_CUPS is not a symlink. Lets fix it." | tee -a ${ERR_LOG}
            createCupsSymLink
        fi
    else
        echo "Not a cluster setup." | tee -a ${ERR_LOG}
    fi
}

DEFAULT_PPD_FILE=/opt/novell/iprintmobile/drivers/default_driver.ppd
DEFAULT_PCL_DRIVER=/opt/novell/iprintmobile/drivers/default_pcl_driver.ppd
updateDrivers()
{
    isCluster
    if [ $? -eq 1 ]; then
        isPoolRunningOnThisNode
        if [ $? -eq 0 ]; then
            echo "Pool is not running on this node. Run this script with -u option on a node where pool is running." | tee -a $ERR_LOG
            return -1
        fi
    fi

    echo "Uploading new generic drivers ... "
    echo "Enter admin user name: (e.g. cn=admin,o=novell)"
    read USERNAME
    echo "Enter password for $USERNAME"
    read -s PASSWD
    echo "Enter Driverstore Host address:"
    read HOSTNAME

    if [ -f $DEFAULT_PPD_FILE ]; then
        # TODO: Export passwd as env variable
        /opt/novell/bin/iprintman driver linux --upload --from-ppd $DEFAULT_PPD_FILE -u $USERNAME -P $PASSWD -s $HOSTNAME --accept-cert --log-level INFO >> $ERR_LOG
    else
        echo "$DEFAULT_PPD_FILE doesn't exist." >> $ERR_LOG
    fi

    if [ -f $DEFAULT_PCL_DRIVER ]; then
        # TODO: Export passwd as env variable
        /opt/novell/bin/iprintman driver linux --upload --from-ppd $DEFAULT_PCL_DRIVER -u $USERNAME -P $PASSWD -s $HOSTNAME --accept-cert --log-level INFO >> $ERR_LOG
    else
        echo "$DEFAULT_PCL_DRIVER doesn't exist." >> $ERR_LOG
    fi
        
    TEST=$(grep "Generic+PostScript+Printer+Foomatic%2FPostscript+%28recommended%29" /etc/cups/printers.conf)
    if [[ -z $TEST ]]; then
        echo "No entry for driver found in cups configuration." >> $ERR_LOG
    else
        /opt/novell/iprintmobile/bin/config_driver.sh -c "Generic+PostScript+Printer+Foomatic%2FPostscript+%28recommended%29" -f $DEFAULT_PPD_FILE -s >> $ERR_LOG
    fi

    TEST=$(grep "Generic+PCL+4+Printer+Foomatic%2Flaserjet+%28recommended%29" /etc/cups/printers.conf)
    if [[ -z $TEST ]]; then
        echo "No entry for driver found in cups configuration." >> $ERR_LOG
    else
        /opt/novell/iprintmobile/bin/config_driver.sh -c "Generic+PCL+4+Printer+Foomatic%2Flaserjet+%28recommended%29" -f $DEFAULT_PCL_DRIVER -s >> $ERR_LOG
    fi
}

fixAll() 
{
    echo "Detect and fix all potential issue"
    # Stop cups before symlink fix
    systemctl stop iprint-cups.service
    fixCupsSymlink
    # Start cups after symlink is fixed
    systemctl start iprint-cups.service
    echo "Updating CUPS drivers"
    updateDrivers
    # Restart cups to pickup new drivers
    systemctl stop iprint-cups.service
    systemctl start iprint-cups.service
}

restartServices()
{
    #(Re)Start CUPS server
    systemctl restart iprint-cups.service
}

usage() {
    echo "This tool is to fix any issue which might be caused by update/upgrade."
    echo "This can be used to fix a specific issue or in auto mode where it can detect"
    echo "the potential issue and try to fix it."
    echo "Usage: $ME <option(s)> "
    echo "-a : Auto mode. Try to detect any potential issue and fix that."
    echo "-c : Fix cups symlink issue"
    echo "-u : Update drivers"
    echo "-r : Restart services"
    echo "-h : display this help screen"
    exit 1	
}

if [ $# -lt 1 ]; then
    usage
fi

FIX_ALL="no"	
FIX_CUPS_SYMLINK="no"
RESTART_SERVICES="no"
UPDATE_DRIVERS="no"
while getopts acruh o
do
    case "$o" in
	    a)	FIX_ALL="yes";;
        c)  FIX_CUPS_SYMLINK="yes";;
        r)  RESTART_SERVICES="yes";;
        u)  UPDATE_DRIVERS="yes";;
        h)  usage;;
		\?) exit 2;;
	esac
done

initLogs
if [ $FIX_ALL == "yes" ]; then
    fixAll
else
    if [ $FIX_CUPS_SYMLINK == "yes" ]; then
        fixCupsSymlink
    fi
    if [ $UPDATE_DRIVERS == "yes" ]; then
        updateDrivers
    fi
fi

if [ "${RESTART_SERVICES}" == "yes" ]; then
    restartServices
fi

exit 0
