#!/bin/sh
#********************************************************************************
# (c) Copyright [2017-2022] Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates 
# and licensors (“Micro Focus”) are as may be set forth in the express warranty 
# statements accompanying such products and services. Nothing herein should be 
# construed as constituting an additional warranty. Micro Focus shall not be liable 
# for technical or editorial errors or omissions contained herein. The information 
# contained herein is subject to change without notice.
#********************************************************************************

# This is a wrapper script meant to be called from yast post config hooks.
# Ideally it should not contain any configuration logic.

ERR_LOG="/var/opt/novell/log/iprintmobile/main_config_oes.log"
LOCAL_ERR_LOG="/var/opt/novell/log/iprintmobile_local/main_config_oes.log"
IPRINT_CONF="/etc/opt/novell/iprint/conf/iprintconf.properties"

ME=$(basename "$0")

initLogs()
{
    if [ ! -d /var/opt/novell/log/iprintmobile/ ]; then
        ERR_LOG=$LOCAL_ERR_LOG
    fi
    echo "Log file: ${ERR_LOG}" >> "${ERR_LOG}"
}

isCluster() 
{
    IPRINT_NSS=$(grep Required-Start /opt/novell/iprint/init.d/novell-ipsmd | grep nss)
    if [ ! -z "${IPRINT_NSS}" ]; then
        echo "Cluster setup" >> "${ERR_LOG}"
        return 1
    else
        echo "Not a cluster setup" >> "${ERR_LOG}"
        return 0
    fi
}

isPoolRunningOnThisNode()
{
    MEDIA_DIR=$(readlink /etc/opt/novell/iprint/conf)
    TEST_FILE="$MEDIA_DIR/iprintconf.properties"
    if [ -f $TEST_FILE ]; then
        echo "Pool is running on this node" >> ${ERR_LOG}
        return 1
    else
        echo "Pool is not running on this node" >> ${ERR_LOG}
        return 0
    fi
}

eDirConfigured()
{
    systemctl status ndsd.service
    if [ $? -eq 0 ];then
        echo "eDirectory is configured and running" >> ${ERR_LOG}
        return 1	
    else
        echo "eDirectory is either not configured or not running." >> "${ERR_LOG}"
        return 0
    fi
}

isMobilityConfigured()
{
    # In case of cluster, we are checking whether pool is running on this node or not.
    # Assumption: Caller takes care of this scenario, Hence the following case might not occur.
    # Side effect: If caller doesn't check for pool before calling this function, caller can
    # start "Full configuration" of Mobile Server(MS) and  further scripts "MIGHT" fail because they will not
    # be able to find the configuration file which are on cluster but are not accessible from this node.  
    if [ ! -f $IPRINT_CONF ]; then
        echo $IPRINT_CONF "is not accessible. Pool might not be running on this node " >> ${ERR_LOG}
        return 0
    fi
    STATUS=$(cat /etc/opt/novell/iprint/conf/iprintconf.properties  | grep mobility_addon_configured | cut -d= -f2)
    if [ $STATUS == "true" ]; then
        echo "Mobility configured" >> "${ERR_LOG}"
        return 1
    else
        echo "Mobility is not configured" >> ${ERR_LOG}
        return 0
    fi
}

startServices()
{
    #(Re)Start Licensing server
    /usr/sbin/rcnovell-iprint-license restart

    #(Re)Start Mobile Server
    /usr/sbin/rcnovell-iprint-mobile restart
}

# This script can be called in two scenarios
# 1. During rpm installation, in this case we need to start services
# 2. When MS is starting and not configured, in this case we MUST not start 
#    services from this script, as it may lead services in endless recursive calls.
# Lets start services when -r is provided else leave it
# Assumption: If -r is not provided script is called from .service file
START_SERVICES="no"
while getopts r o
do
    case "$o" in
	    r)	START_SERVICES="yes";;
		\?)     usage;;
	esac
done

initLogs
isCluster
if [[ $? -eq 1 ]]; then
    isPoolRunningOnThisNode
    if [[ $? -eq 1 ]]; then
        echo "Pool is running on this node" >> ${ERR_LOG}
        isMobilityConfigured
        if [[ $? -eq 0 ]]; then	
            eDirConfigured
            if [[ $? -eq 1 ]]; then
                echo "eDir is configured. Lets proceed with configuration." >> ${ERR_LOG}
                /usr/bin/sh /opt/novell/iprintmobile/bin/yast_mobile_config.sh -a
                
                if [ "${START_SERVICES}" == "yes" ]; then
                    echo "Starting services" >> ${ERR_LOG}
                    startServices
                fi
                exit 0
            else
                echo "eDir is not configured on this machine. Lets wait till this script is called next time." >> ${ERR_LOG}
                exit 0
            fi
        else
            echo "Mobility is already configured. No work for me." >> ${ERR_LOG}
            exit 0
        fi
    else
        echo "Pool is not running on this node" >> ${ERR_LOG}
        echo "Creating certificates" >> ${ERR_LOG}
        /usr/bin/sh /opt/novell/iprintmobile/bin/yast_mobile_config.sh -c
        echo "New certificate will be used by service when it gets started as part of pool migration." >> ${ERR_LOG}
        exit 0
    fi
else
    echo "This is a standalone node. Lets check if it is ready for configuration." >> ${ERR_LOG}
    isMobilityConfigured
    if [[ $? -eq 0 ]]; then
        echo "Mobility is not configured. Checking for eDir configuration." >> ${ERR_LOG} 
        eDirConfigured
        if [[ $? -eq 1 ]]; then
            echo "eDir is configured. Lets proceed with configuration." >> ${ERR_LOG}
            /usr/bin/sh /opt/novell/iprintmobile/bin/yast_mobile_config.sh -a

            if [ "${START_SERVICES}" == "yes" ]; then
                echo "Starting services" >> ${ERR_LOG}
                startServices
            fi
            exit 0
        else
            echo "eDir is not configured on this machine. Lets wait till this script is called next time." >> ${ERR_LOG}
            exit 0
        fi
    else
        echo "Mobility is already configured. Certs should be created." >> ${ERR_LOG}
		echo "Creating certificates" >> ${ERR_LOG}
		/usr/bin/sh /opt/novell/iprintmobile/bin/yast_mobile_config.sh -c
        exit 0
    fi
fi

echo "This script should never reach here. In a worst case if script reaches here." >> ${ERR_LOG}
echo "Lets still return success so that wrapper scripts/rpm should not fail." >> ${ERR_LOG}
exit 0

