#!/bin/sh
#****************************************************************************
# Copyright 2017-2023 Open Text.
#
# The only warranties for products and services of Open Text and its affiliates and licensors (“Open Text”)
# are as may be set forth in the express warranty statements accompanying such products and services.
# Nothing herein should be construed as constituting an additional warranty. Open Text shall not be liable
# for technical or editorial errors or omissions contained herein. The information contained herein is
# subject to change without notice.
#****************************************************************************

#Include library
. /opt/novell/iprintmobile/bin/lib.sh

CERT_DIRECTORY="/etc/opt/novell/iprintmobile/certs"
CLIENT_CERT_DIRECTORY=${CERT_DIRECTORY}/clientcerts

SERVER_CERT="/etc/ssl/servercerts/servercert.pem"
SERVER_KEY="/etc/ssl/servercerts/serverkey.pem"
SERVER_CERT_PFX=$CERT_DIRECTORY/servercert.pfx
KEYSTORE=$CERT_DIRECTORY/keystore
KEY_PASSWORD="changeit"
ROOT_CA_CRT="/etc/opt/novell/certs/SSCert.pem"
ROOT_CA_ALIAS="org_ca"

IPRINT_SSL_CONF="/etc/opt/novell/iprint/httpd/conf/iprint_ssl.conf"
IPRINT_LICENSE="/etc/opt/novell/iprint/licenseKeys"
IPRINT_CONF="/etc/opt/novell/iprintmobile/conf/"
IPRINT_MOBILE_CONF="/etc/opt/novell/iprintmobile/conf/iprintmobile.conf"
ERR_LOG="/var/opt/novell/log/iprintmobile/config_oes.log"

ME=$(basename "$0")

init()
{
    if [ ! -d /var/opt/novell/log/iprintmobile/ ]; then
       ERR_LOG="/var/opt/novell/log/iprintmobile_local/main_config_oes.log"
    fi
    logInfo "Initializing log file: ${ERR_LOG}" 
    logInfo "Creating cert directory for mobile server"
    if [ ! -d $CERT_DIRECTORY ]; then
        mkdir -p $CERT_DIRECTORY
    fi    
}

logError()
{
    echo "ERROR: "$1 | tee -a $ERR_LOG
}

logWarning()
{
    echo "WARNING: "$1 | tee -a $ERR_LOG
}

logInfo()
{
    echo "INFO: "$1 | tee -a $ERR_LOG
}

createKeystore()
{

    if [ ! -f $SERVER_CERT ] || [ ! -f $SERVER_KEY ]; then
	logError "Server certificate and/or key file doesn't exist. Exiting..."
	exit 1
    fi
	
    logInfo "Converting pem to pfx"
    # Convert pem to pfx 
    $OPENSSL pkcs12 -export -in $SERVER_CERT -inkey $SERVER_KEY  -out $SERVER_CERT_PFX -passin pass:$KEY_PASSWORD -passout pass:$KEY_PASSWORD
    if [ $? != 0 ]; then
        logError "Failed to convert pem to pfx"
        exit 2
    else
        logInfo "Pem to pfx converted successfully"
    fi    

    logInfo "Importing pfx to keystore"
    $KEYTOOL -importkeystore -srckeystore $SERVER_CERT_PFX -srcstoretype pkcs12 -destkeystore $KEYSTORE -deststoretype JKS -srcstorepass $KEY_PASSWORD -deststorepass $KEY_PASSWORD -noprompt 
    if [ $? != 0 ]; then
        logError "Failed to import pfx cert into java keystore"
        exit 3
    else
        logInfo "pfx cert imported into java keystore successfully"
    fi

	#deletes the old root ca alias
	deleteKeyWithAlias "mykey"
	deleteKeyWithAlias $ROOT_CA_ALIAS
    logInfo "Importing root CA to keystore"
    $KEYTOOL -import -trustcacerts -file $ROOT_CA_CRT -keystore $KEYSTORE -storepass $KEY_PASSWORD -alias $ROOT_CA_ALIAS -noprompt
    if [ $? != 0 ]; then
        logError "Failed to import root cert into java keystore"
        exit 3
    else
        logInfo "root cert imported into java keystore successfully"
    fi    
}

addAuthUrl()
{
	if [ -f ${IPRINT_SSL_CONF} ]; then
		sed -i '/<LocationMatch/c \\t<LocationMatch ^/(ipps|idss|[Pp][Ss][Mm][Ss][Tt][Aa][Tt][Uu][Ss]|auth)($|/)>' ${IPRINT_SSL_CONF}
	else
		logError "${IPRINT_SSL_CONF} doesn't exist. Exiting..."
		exit 4
	fi	
}

setPermissions ()
{
	if [ -d ${IPRINT_LICENSE} ]; then
	    if [ $(getent group licenseOwners) ]; then
	        logInfo "licenseOwners group already exist"
	    else
	        groupadd licenseOwners -g 211
	        retval=$?
	        if [ $retval -eq 0 ]; then
	            logInfo "Added group licenseOwners"
	        else
	            logError "Problem adding group licenseOwners : $retval"
	        fi
	    fi

		usermod -G licenseOwners -a root
		retval=$?
		if [ $retval -eq 0 ]; then
			logInfo "Added root to licenseOwners"
		else
			logError "Problem adding root to licenseOwners : $retval"
		fi
		usermod -G licenseOwners -a iprint
		retval=$?
		if [ $retval -eq 0 ]; then
			logInfo "Added iprint to licenseOwners"
		else
			logError "Problem adding iprint to licenseOwners : $retval"
		fi
		usermod -G licenseOwners -a iprintmobile
		retval=$?
		if [ $retval -eq 0 ]; then
			logInfo "Added iprintmobile to licenseOwners"
		else
			logError "Problem adding iprintmobile to licenseOwners : $retval"
		fi
		chgrp licenseOwners ${IPRINT_LICENSE}/
		retval=$?
		if [ $retval -eq 0 ]; then
			logInfo "changed group licenseOwners"
		else
			logError "Problem adding changing group to licenseOwners : $retval"
		fi
		chgrp licenseOwners ${IPRINT_LICENSE}/license.xml
		retval=$?
		if [ $retval -eq 0 ]; then
			logInfo "changed group licenseOwners"
		else
			logError "Problem adding changing group to licenseOwners : $retval"
		fi
		chmod g+rw ${IPRINT_LICENSE}/
		retval=$?
		if [ $retval -eq 0 ]; then
		logInfo "g+rw group licenseOwners"
		else
			logError "Problem adding g/=rw group to licenseOwners : $retval"
		fi
		chmod g+rw ${IPRINT_LICENSE}/license.xml
		retval=$?
		if [ $retval -eq 0 ]; then
		logInfo "g+rw group licenseOwners license"
		else
			logError "Problem adding g/=rw group to license.xml licenseOwners : $retval"
		fi
	else
		logError  "${IPRINT_LICENSE} not found."
	fi
	
	if [ -d ${IPRINT_CONF} ]; then
		groupadd confFileOwners -g 212
		usermod -G confFileOwners -a root
		usermod -G confFileOwners -a iprintmobile
		chgrp confFileOwners ${IPRINT_CONF}
		chmod g+rw ${IPRINT_CONF}
		
	else
		logError "${IPRINT_CONF} not found."
	fi
	
}

setiPrintMobileConfPermission ()
{
	if [ -f ${IPRINT_MOBILE_CONF} ];then
		chgrp confFileOwners ${IPRINT_MOBILE_CONF}
		chmod g+rw ${IPRINT_MOBILE_CONF}
	else
		logError "${IPRINT_MOBILE_CONF} not found."
	fi
}

#Deletes the key with alias from keystore
deleteKeyWithAlias()
{
    ALIAS=$1

    $KEYTOOL -list -v -keystore $KEYSTORE -alias $ALIAS -storePass $KEY_PASSWORD -noprompt >> /dev/null
    if  [ $? == 0 ]; then
        logInfo "Key with alias '$ALIAS' exists. deleting it"
        $KEYTOOL -delete -alias $ALIAS -keystore $KEYSTORE -storePass $KEY_PASSWORD -noprompt
    fi

}

usage() 
{
    echo "Usage:"
    echo "sh ${ME} <option(s)>"
    echo "valid options: "
    echo "-a : Perform a full configuration"
    echo "-c : Create certificates only"
    exit 0
}

#Execution will start from this point.
if [ "$#" -lt 1 ]; then
    usage
fi

CREATE_CERTS_ONLY="no"
DO_ALL="no"
while getopts ca o
do
    case "$o" in
	    c)	CREATE_CERTS_ONLY="yes";;
		a)	DO_ALL="yes";;
		\?)     usage;;
	esac
done

# Init log file and create cert directory if it donesn't exist
init

# If we are performing a full configuration, skip other command line parameters
if [ "${DO_ALL}" == "no" ]; then
	logInfo "Script is asked not do complete configuration."
	if [ "${CREATE_CERTS_ONLY}" == "yes" ]; then
		logInfo "Lets create certificate only and get out of here"
		logInfo "Creating keystore for mobile server..."
		createKeystore
		logInfo "Updating iprintmobile user privileges in /etc/sudoers"
		/usr/sbin/iprintmobile_config --assign-permission
		exit 0
	fi
fi

logInfo "Creating keystore for mobile server"
createKeystore

logInfo "Adding /auth url"
addAuthUrl

logInfo "Setting Permissions"
setPermissions

logInfo "Starting database"
/usr/bin/sh /usr/share/orientdb/bin/config_db.sh

logInfo "Starting Auth service"
systemctl enable --now iprint-auth.service

logInfo "Configuring iPrint Mobile Service "  
/usr/sbin/iprintmobile_config --defaultconfig

logInfo "Setting iprintmobileConf permission "
setiPrintMobileConfPermission
exit 0 
